import "package:test/test.dart";

import "package:gql/ast.dart";
import "package:gql/language.dart";

NameNode _name(String value) => NameNode(value: value);

void main() {
  group("Printer", () {
    group("values", () {
      test("BooleanValueNode true", () {
        expect(
          printNode(
            BooleanValueNode(value: true),
          ),
          "true",
        );
      });

      test("BooleanValueNode false", () {
        expect(
          printNode(
            BooleanValueNode(value: false),
          ),
          "false",
        );
      });

      test("NullValueNode", () {
        expect(
          printNode(
            NullValueNode(),
          ),
          "null",
        );
      });

      test("EnumValueNode", () {
        expect(
          printNode(
            EnumValueNode(
              name: _name("ON"),
            ),
          ),
          "ON",
        );
      });

      test("FloatValueNode", () {
        expect(
          printNode(
            FloatValueNode(value: "1.23"),
          ),
          "1.23",
        );
      });

      test("IntValueNode", () {
        expect(
          printNode(
            IntValueNode(value: "123"),
          ),
          "123",
        );
      });

      test("StringValueNode", () {
        expect(
          printNode(
            StringValueNode(
              value: "123",
              isBlock: false,
            ),
          ),
          "123",
        );
      });
      test("StringValueNode block", () {
        expect(
          printNode(
            StringValueNode(
              value: "123",
              isBlock: true,
            ),
          ),
          "123",
        );
      });

      test("ListValueNode empty", () {
        expect(
          printNode(
            ListValueNode(
              values: [],
            ),
          ),
          "[]",
        );
      });

      test("ListValueNode", () {
        expect(
          printNode(
            ListValueNode(
              values: [
                IntValueNode(value: "123"),
                FloatValueNode(value: "1.23"),
              ],
            ),
          ),
          "[123, 1.23]",
        );
      });

      test("ObjectValueNode empty", () {
        expect(
          printNode(
            ObjectValueNode(
              fields: [],
            ),
          ),
          "{}",
        );
      });

      test("ObjectValueNode", () {
        expect(
          printNode(
            ObjectValueNode(
              fields: [
                ObjectFieldNode(
                  name: _name("boolA"),
                  value: BooleanValueNode(value: true),
                ),
                ObjectFieldNode(
                  name: _name("boolB"),
                  value: BooleanValueNode(value: false),
                ),
              ],
            ),
          ),
          "{boolA: true, boolB: false}",
        );
      });
    });
  });
}
